/**
 * This handles the sg_youtube and sg_video read more button
 */
export default class SgVideo {
	constructor(element, settings) {
		this.settings = settings;
		this.dom = {
			list: element,
			listItems: element.querySelectorAll('.sg-video__list-item'),
		};

		this.active = 0;

		this.dom.listItems.forEach((item, index) => {
			this.setupReadMore(index);
		});

		if (!this.settings.disableMinHeight) {
			this.checkImageSizes();
		}
	}

	/**
	 * Initialize the default sg video objects
	 */
	static initDefault() {
		/* eslint no-new: "off" */
		document.querySelectorAll('.sg-video__list--default').forEach((item) => {
			new SgVideo(item, {
				visibleLines: 10,
				textSelector: '.sg-video__bodytext',
				type: 'default',
			});
		});

		document.querySelectorAll('.sg-video__list--rows').forEach((item) => {
			new SgVideo(item, { textSelector: '.sg-video__description' });
		});
	}

	/**
	 * The largest image height will be applied to the other images
	 */
	checkImageSizes() {
		let highestValue = 0;
		const images = [];
		this.dom.listItems.forEach((item) => {
			const image = item.querySelector('img');
			if (image && image.height > highestValue) {
				highestValue = image.height;
			}
			images.push(image);
		});

		images.forEach((image) => {
			image.style.minHeight = `${highestValue}px`;
		});
	}

	setupReadMore(index) {
		const item = this.dom.listItems[index];
		const button = item.querySelector('.sg-video__read-more');
		const text = item.querySelector(this.settings.textSelector);
		let visibleLines = 4;
		if (this.settings.visibleLines) {
			visibleLines = this.settings.visibleLines;
		}

		if (!text) {
			if (button) {
				button.classList.add('disabled');
			}
			return;
		}

		if (!Object.prototype.hasOwnProperty.call(this.settings, 'itemHeight')) {
			this.settings.itemHeight =
				Number.parseFloat(
					window.getComputedStyle(text, null).getPropertyValue('line-height'),
				) * visibleLines;
		}

		if (!button) {
			return;
		}

		if (!text || !window.matchMedia('(min-width: 1225px)').matches) {
			button.classList.add('disabled');
			return;
		}

		const textHeight = text.offsetHeight;
		text.dataset.height = textHeight;

		// Check if the the is too small and hide the "Read more" button
		text.style.maxHeight = `${this.settings.itemHeight}px`;
		if (this.isTextShort(textHeight)) {
			button.classList.add('disabled');
			return;
		}

		button.addEventListener('click', () => this.showText(index));
	}

	showText(index) {
		const item = this.dom.listItems[index];

		if (item.classList.contains('expanded')) {
			this.active -= 1;
			this.hideText(index);
			return;
		}

		this.active += 1;
		const button = item.querySelector('.sg-video__read-more');
		const text = item.querySelector(this.settings.textSelector);

		// Do not add expanded if the screen is too small
		if (this.settings.type === 'default') {
			this.dom.listItems.forEach((_item) => {
				_item.style.height = `${_item.getBoundingClientRect().height}px`;
			});
		}
		item.classList.add('expanded');
		text.classList.add('expanded');
		item.style.zIndex = 10 + (this.dom.listItems.length - index);
		text.style.maxHeight = `${text.dataset.height}px`;
		button.classList.add('expanded');
		button.textContent = button.dataset.buttonCloseText;
	}

	hideText(index) {
		const item = this.dom.listItems[index];
		const button = item.querySelector('.sg-video__read-more');
		const text = item.querySelector(this.settings.textSelector);

		text.style.maxHeight = `${this.settings.itemHeight}px`;
		text.classList.remove('expanded');
		button.classList.remove('expanded');
		button.textContent = button.dataset.buttonOpenText;
		setTimeout(() => {
			if (this.settings.type === 'default' && this.active === 0) {
				this.dom.listItems.forEach((_item) => {
					_item.style.height = '';
					_item.style.zIndex = '';
				});
			}
			item.classList.remove('expanded');
		}, 200);
	}

	isTextShort(height) {
		// Add the padding of the bottom button
		return height <= this.settings.itemHeight + 20;
	}
}
