import * as BasicLightbox from 'basiclightbox';

export default class SgVideoLightbox {
	/**
	 * Initializes the LightboxManager with the necessary parameters.
	 */
	constructor() {
		const videoItems = document.querySelectorAll('.sg-video-item');
		const isMobile = window.matchMedia('(max-width: 679px)').matches;

		videoItems.forEach((item) => {
			if (
				(item.dataset.disableLightboxMobile === '1' && isMobile) ||
				(item.dataset.disableLightbox === '1' && !isMobile)
			) {
				item.classList.remove('sg-video-item');
				item.addEventListener('click', SgVideoLightbox.disableLightbox.bind(this));
			}
		});

		Array.prototype.forEach.call(document.querySelectorAll('.sg-video-item'), (element) => {
			element.addEventListener('click', SgVideoLightbox.openLightbox);
		});
	}

	static openLightbox(event) {
		event.preventDefault();
		switch (event.target.closest('a').dataset.videoType) {
			case 'youtube': {
				SgVideoLightbox.openYouTubeLightBox(event);
				break;
			}
			case 'vimeo': {
				SgVideoLightbox.openVimeoLightBox(event);
				break;
			}
			default:
			// do nothing
		}
	}

	static openVimeoLightBox(event) {
		event.doSomething();
	}

	static openYouTubeLightBox(event) {
		let url = event.target.closest('a').href;
		const videoId = SgVideoLightbox.getVideoIdFromUrl(url);
		url = `https://www.youtube-nocookie.com/embed/${videoId}`;

		const instance = BasicLightbox.create(
			`<div class="sg-video-lightbox-wrapper">
				<iframe allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" 
				class="sg-video-iframe sg-video-youtube-iframe" src="${url}"></iframe>
			</div>
		`,
			{
				closable: false,
				onClose: (inst, event) => {
					console.log(inst, event); // eslint-disable-line no-console
					debugger; // eslint-disable-line no-debugger
				},
			},
		);

		SgVideoLightbox.resizeYoutubeIframe(instance.element().querySelector('iframe'));

		instance.show();
	}

	static resizeYoutubeIframe(iframe) {
		const width = window.innerWidth * 0.6; // set width to 60% of window width
		const height = width * 0.5625; // set height to maintain 16:9 aspect ratio
		iframe.width = Number.parseInt(width); // set the iframe width
		iframe.height = Number.parseInt(height); // set the iframe height
		iframe.parentElement.style.width = `${width}px`;
		iframe.parentElement.style.height = `${height}px`;
	}

	/**
	 * Replace the image with an iframe
	 *
	 * @param event
	 */
	static disableLightbox(event) {
		event.preventDefault();
		const item = event.currentTarget;
		item.classList.add('no-lightbox');

		const videoId = SgVideoLightbox.includeAdditionalUrlParameters(
			SgVideoLightbox.getVideoIdFromUrl(item.href),
			item.dataset.additionalUrlParameters,
		);
		const videoImage = item.querySelector('.sg-video__image');
		const height = videoImage.offsetHeight;
		const width = videoImage.offsetWidth;

		const iframe = document.createElement('iframe');
		iframe.width = width;
		iframe.height = height;
		iframe.style.border = 'none';
		iframe.allowFullscreen = true;
		iframe.allow =
			'accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture';
		iframe.src = `https://www.youtube-nocookie.com/embed/${videoId}`;

		if (videoImage.parentElement.nodeName.toLowerCase() === 'picture') {
			item.replaceChild(iframe, videoImage.parentElement);
		} else {
			videoImage.replaceWith(iframe);
		}
	}

	/**
	 * Prepares the given url, or returns null.
	 *
	 * @param {string} url
	 * @return {string|null}
	 */
	static getVideoIdFromUrl(url) {
		let matches = url.match(/watch\?v=(.*)&list=(.*)/);
		if (!matches) {
			// check if the list parameter is missing
			matches = url.match(/watch\?v=([^&?]*)/);
			if (!matches) {
				return null;
			}
		}
		let [, videoString] = matches;
		let queryParameterSeparator = '?';
		if (matches[2]) {
			videoString += `?list=${matches[2]}`;
			queryParameterSeparator = '&';
		}

		return `${videoString + queryParameterSeparator}autoplay=1&rel=0`;
	}

	/**
	 * Includes the additional URL parameters in the video url
	 *
	 * @param {string|null} url
	 * @param {string} _additionalUrlParameters
	 * @returns
	 */
	static includeAdditionalUrlParameters(url, _additionalUrlParameters = '') {
		if (!url) {
			return '';
		}

		if (!_additionalUrlParameters) {
			return url;
		}

		let additionalUrlParameters = _additionalUrlParameters;
		const beginsWithQuestionMark = additionalUrlParameters.charAt(0) === '?';
		const beginsWithAmpersand = additionalUrlParameters.charAt(0) === '&';

		if (beginsWithQuestionMark || beginsWithAmpersand) {
			additionalUrlParameters = additionalUrlParameters.slice(1);
		}

		return url.includes('?')
			? `${url}&${additionalUrlParameters}`
			: `${url}?${additionalUrlParameters}`;
	}
}
